@@ -1,6 +1,5 @@
 /* GStreamer
  * Copyright (C) 2011 David Schleef <ds@entropywave.com>
- * Copyright (C) 2021 Igalia S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,20 +18,20 @@
  */
 /**
  * SECTION:element-gstsouphttpclientsink
- * @title: gstsouphttpclientsink
  *
  * The souphttpclientsink element sends pipeline data to an HTTP server
  * using HTTP PUT commands.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 -v videotestsrc num-buffers=300 ! theoraenc ! oggmux !
  *   souphttpclientsink location=http://server/filename.ogv
  * ]|
- *
+ * 
  * This example encodes 10 seconds of video and sends it to the HTTP
  * server "server" using HTTP PUT commands.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,12 +40,11 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstbasesink.h>
-#include <gio/gio.h>
-
-#include "gstsoupelements.h"
 #include "gstsouphttpclientsink.h"
 #include "gstsouputils.h"
 
+#include <gst/glib-compat-private.h>
+
 GST_DEBUG_CATEGORY_STATIC (souphttpclientsink_dbg);
 #define GST_CAT_DEFAULT souphttpclientsink_dbg
 
@@ -62,23 +60,27 @@ static void gst_soup_http_client_sink_finalize (GObject * object);
 
 static gboolean gst_soup_http_client_sink_set_caps (GstBaseSink * sink,
     GstCaps * caps);
+static void gst_soup_http_client_sink_get_times (GstBaseSink * sink,
+    GstBuffer * buffer, GstClockTime * start, GstClockTime * end);
 static gboolean gst_soup_http_client_sink_start (GstBaseSink * sink);
 static gboolean gst_soup_http_client_sink_stop (GstBaseSink * sink);
 static gboolean gst_soup_http_client_sink_unlock (GstBaseSink * sink);
+static gboolean gst_soup_http_client_sink_event (GstBaseSink * sink,
+    GstEvent * event);
+static GstFlowReturn gst_soup_http_client_sink_preroll (GstBaseSink * sink,
+    GstBuffer * buffer);
 static GstFlowReturn gst_soup_http_client_sink_render (GstBaseSink * sink,
     GstBuffer * buffer);
+
+static void free_buffer_list (GList * list);
 static void gst_soup_http_client_sink_reset (GstSoupHttpClientSink *
     souphttpsink);
-
-static gboolean authenticate (SoupMessage * msg, SoupAuth * auth,
-    gboolean retrying, gpointer user_data);
-static void restarted (SoupMessage * msg, GBytes * body);
-static gboolean send_handle_status (SoupMessage * msg, GError * error,
-    GstSoupHttpClientSink * sink);
-
-static gboolean
-gst_soup_http_client_sink_set_proxy (GstSoupHttpClientSink * souphttpsink,
-    const gchar * uri);
+static void authenticate (SoupSession * session, SoupMessage * msg,
+    SoupAuth * auth, gboolean retrying, gpointer user_data);
+static void callback (SoupSession * session, SoupMessage * msg,
+    gpointer user_data);
+static gboolean gst_soup_http_client_sink_set_proxy (GstSoupHttpClientSink *
+    souphttpsink, const gchar * uri);
 
 enum
 {
@@ -93,9 +95,7 @@ enum
   PROP_PROXY_PW,
   PROP_COOKIES,
   PROP_SESSION,
-  PROP_SOUP_LOG_LEVEL,
-  PROP_RETRY_DELAY,
-  PROP_RETRIES
+  PROP_SOUP_LOG_LEVEL
 };
 
 #define DEFAULT_USER_AGENT           "GStreamer souphttpclientsink "
@@ -116,11 +116,6 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 G_DEFINE_TYPE (GstSoupHttpClientSink, gst_soup_http_client_sink,
     GST_TYPE_BASE_SINK);
 
-static gboolean souphttpclientsink_element_init (GstPlugin * plugin);
-GST_ELEMENT_REGISTER_DEFINE_CUSTOM (souphttpclientsink,
-    souphttpclientsink_element_init);
-
-
 static void
 gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
 {
@@ -172,19 +167,10 @@ gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_SESSION,
       g_param_spec_object ("session", "session",
           "SoupSession object to use for communication",
-          _soup_session_get_type (),
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          SOUP_TYPE_SESSION, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_COOKIES,
       g_param_spec_boxed ("cookies", "Cookies", "HTTP request cookies",
           G_TYPE_STRV, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_RETRY_DELAY,
-      g_param_spec_int ("retry-delay", "Retry Delay",
-          "Delay in seconds between retries after a failure", 1, G_MAXINT, 5,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_RETRIES,
-      g_param_spec_int ("retries", "Retries",
-          "Maximum number of retries, zero to disable, -1 to retry forever",
-          -1, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
    * GstSoupHttpClientSink::http-log-level:
    *
@@ -196,11 +182,11 @@ gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_SOUP_LOG_LEVEL,
       g_param_spec_enum ("http-log-level", "HTTP log level",
           "Set log level for soup's HTTP session log",
-          _soup_logger_log_level_get_type (),
-          DEFAULT_SOUP_LOG_LEVEL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          SOUP_TYPE_LOGGER_LOG_LEVEL, DEFAULT_SOUP_LOG_LEVEL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_soup_http_client_sink_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_soup_http_client_sink_sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class, "HTTP client sink",
       "Generic", "Sends streams to HTTP server via PUT",
@@ -208,12 +194,23 @@ gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
 
   base_sink_class->set_caps =
       GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_set_caps);
+  if (0)
+    base_sink_class->get_times =
+        GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_get_times);
   base_sink_class->start = GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_start);
   base_sink_class->stop = GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_stop);
   base_sink_class->unlock =
       GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_unlock);
+  base_sink_class->event = GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_event);
+  if (0)
+    base_sink_class->preroll =
+        GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_preroll);
   base_sink_class->render =
       GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_render);
+
+  GST_DEBUG_CATEGORY_INIT (souphttpclientsink_dbg, "souphttpclientsink", 0,
+      "souphttpclientsink element");
+
 }
 
 static void
@@ -234,9 +231,6 @@ gst_soup_http_client_sink_init (GstSoupHttpClientSink * souphttpsink)
   souphttpsink->prop_session = NULL;
   souphttpsink->timeout = 1;
   souphttpsink->log_level = DEFAULT_SOUP_LOG_LEVEL;
-  souphttpsink->retry_delay = 5;
-  souphttpsink->retries = 0;
-  souphttpsink->sent_buffers = NULL;
   proxy = g_getenv ("http_proxy");
   if (proxy && !gst_soup_http_client_sink_set_proxy (souphttpsink, proxy)) {
     GST_WARNING_OBJECT (souphttpsink,
@@ -250,21 +244,11 @@ gst_soup_http_client_sink_init (GstSoupHttpClientSink * souphttpsink)
 static void
 gst_soup_http_client_sink_reset (GstSoupHttpClientSink * souphttpsink)
 {
-  g_list_free_full (souphttpsink->queued_buffers,
-      (GDestroyNotify) gst_buffer_unref);
-  souphttpsink->queued_buffers = NULL;
   g_free (souphttpsink->reason_phrase);
   souphttpsink->reason_phrase = NULL;
   souphttpsink->status_code = 0;
   souphttpsink->offset = 0;
-  souphttpsink->failures = 0;
 
-  g_list_free_full (souphttpsink->streamheader_buffers,
-      (GDestroyNotify) gst_buffer_unref);
-  souphttpsink->streamheader_buffers = NULL;
-  g_list_free_full (souphttpsink->sent_buffers,
-      (GDestroyNotify) gst_buffer_unref);
-  souphttpsink->sent_buffers = NULL;
 }
 
 static gboolean
@@ -272,15 +256,15 @@ gst_soup_http_client_sink_set_proxy (GstSoupHttpClientSink * souphttpsink,
     const gchar * uri)
 {
   if (souphttpsink->proxy) {
-    gst_soup_uri_free (souphttpsink->proxy);
+    soup_uri_free (souphttpsink->proxy);
     souphttpsink->proxy = NULL;
   }
   if (g_str_has_prefix (uri, "http://")) {
-    souphttpsink->proxy = gst_soup_uri_new (uri);
+    souphttpsink->proxy = soup_uri_new (uri);
   } else {
     gchar *new_uri = g_strconcat ("http://", uri, NULL);
 
-    souphttpsink->proxy = gst_soup_uri_new (new_uri);
+    souphttpsink->proxy = soup_uri_new (new_uri);
     g_free (new_uri);
   }
 
@@ -305,14 +289,6 @@ gst_soup_http_client_sink_set_property (GObject * object, guint property_id,
       g_free (souphttpsink->location);
       souphttpsink->location = g_value_dup_string (value);
       souphttpsink->offset = 0;
-      if ((souphttpsink->location == NULL)
-          || !gst_uri_is_valid (souphttpsink->location)) {
-        GST_WARNING_OBJECT (souphttpsink,
-            "The location (\"%s\") set, is not a valid uri.",
-            souphttpsink->location);
-        g_free (souphttpsink->location);
-        souphttpsink->location = NULL;
-      }
       break;
     case PROP_USER_AGENT:
       g_free (souphttpsink->user_agent);
@@ -360,12 +336,6 @@ gst_soup_http_client_sink_set_property (GObject * object, guint property_id,
     case PROP_SOUP_LOG_LEVEL:
       souphttpsink->log_level = g_value_get_enum (value);
       break;
-    case PROP_RETRY_DELAY:
-      souphttpsink->retry_delay = g_value_get_int (value);
-      break;
-    case PROP_RETRIES:
-      souphttpsink->retries = g_value_get_int (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -409,7 +379,7 @@ gst_soup_http_client_sink_get_property (GObject * object, guint property_id,
       if (souphttpsink->proxy == NULL)
         g_value_set_static_string (value, "");
       else {
-        char *proxy = gst_soup_uri_to_string (souphttpsink->proxy);
+        char *proxy = soup_uri_to_string (souphttpsink->proxy, FALSE);
 
         g_value_set_string (value, proxy);
         g_free (proxy);
@@ -421,12 +391,6 @@ gst_soup_http_client_sink_get_property (GObject * object, guint property_id,
     case PROP_SOUP_LOG_LEVEL:
       g_value_set_enum (value, souphttpsink->log_level);
       break;
-    case PROP_RETRY_DELAY:
-      g_value_set_int (value, souphttpsink->retry_delay);
-      break;
-    case PROP_RETRIES:
-      g_value_set_int (value, souphttpsink->retries);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -459,9 +423,8 @@ gst_soup_http_client_sink_finalize (GObject * object)
   g_free (souphttpsink->proxy_id);
   g_free (souphttpsink->proxy_pw);
   if (souphttpsink->proxy)
-    gst_soup_uri_free (souphttpsink->proxy);
+    soup_uri_free (souphttpsink->proxy);
   g_free (souphttpsink->location);
-  g_strfreev (souphttpsink->cookies);
 
   g_cond_clear (&souphttpsink->cond);
   g_mutex_clear (&souphttpsink->mutex);
@@ -469,6 +432,8 @@ gst_soup_http_client_sink_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+
+
 static gboolean
 gst_soup_http_client_sink_set_caps (GstBaseSink * sink, GstCaps * caps)
 {
@@ -477,12 +442,10 @@ gst_soup_http_client_sink_set_caps (GstBaseSink * sink, GstCaps * caps)
   const GValue *value_array;
   int i, n;
 
-  GST_DEBUG_OBJECT (souphttpsink, "new stream headers set");
   structure = gst_caps_get_structure (caps, 0);
   value_array = gst_structure_get_value (structure, "streamheader");
   if (value_array) {
-    g_list_free_full (souphttpsink->streamheader_buffers,
-        (GDestroyNotify) gst_buffer_unref);
+    free_buffer_list (souphttpsink->streamheader_buffers);
     souphttpsink->streamheader_buffers = NULL;
 
     n = gst_value_array_get_size (value_array);
@@ -500,6 +463,13 @@ gst_soup_http_client_sink_set_caps (GstBaseSink * sink, GstCaps * caps)
   return TRUE;
 }
 
+static void
+gst_soup_http_client_sink_get_times (GstBaseSink * sink, GstBuffer * buffer,
+    GstClockTime * start, GstClockTime * end)
+{
+
+}
+
 static gboolean
 thread_ready_idle_cb (gpointer data)
 {
@@ -518,38 +488,11 @@ static gpointer
 thread_func (gpointer ptr)
 {
   GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (ptr);
-  GProxyResolver *proxy_resolver;
-  GMainContext *context;
 
   GST_DEBUG ("thread start");
 
-  context = souphttpsink->context;
-  g_main_context_push_thread_default (context);
-
-  if (souphttpsink->proxy != NULL) {
-    char *proxy_string = gst_soup_uri_to_string (souphttpsink->proxy);
-    proxy_resolver = g_simple_proxy_resolver_new (proxy_string, NULL);
-    g_free (proxy_string);
-  } else
-    proxy_resolver = g_object_ref (g_proxy_resolver_get_default ());
-
-  souphttpsink->session =
-      _soup_session_new_with_options ("user-agent", souphttpsink->user_agent,
-      "timeout", souphttpsink->timeout, "proxy-resolver", proxy_resolver, NULL);
-
-  g_object_unref (proxy_resolver);
-
-  if (gst_soup_loader_get_api_version () < 3) {
-    g_signal_connect (souphttpsink->session, "authenticate",
-        G_CALLBACK (authenticate), souphttpsink);
-  }
-
-  GST_DEBUG ("created session");
-
   g_main_loop_run (souphttpsink->loop);
 
-  g_main_context_pop_thread_default (context);
-
   GST_DEBUG ("thread quit");
 
   return NULL;
@@ -575,31 +518,32 @@ gst_soup_http_client_sink_start (GstBaseSink * sink)
     g_source_attach (source, souphttpsink->context);
     g_source_unref (source);
 
-    souphttpsink->loop = g_main_loop_new (souphttpsink->context, FALSE);
+    souphttpsink->loop = g_main_loop_new (souphttpsink->context, TRUE);
 
     g_mutex_lock (&souphttpsink->mutex);
 
+    /* FIXME: error handling */
     souphttpsink->thread = g_thread_try_new ("souphttpclientsink-thread",
         thread_func, souphttpsink, &error);
 
-    if (error != NULL) {
-      GST_DEBUG_OBJECT (souphttpsink, "failed to start thread, %s",
-          error->message);
-      g_error_free (error);
-      g_mutex_unlock (&souphttpsink->mutex);
-      return FALSE;
-    }
-
     GST_LOG_OBJECT (souphttpsink, "waiting for main loop thread to start up");
-    while (!g_main_loop_is_running (souphttpsink->loop))
-      g_cond_wait (&souphttpsink->cond, &souphttpsink->mutex);
+    g_cond_wait (&souphttpsink->cond, &souphttpsink->mutex);
     g_mutex_unlock (&souphttpsink->mutex);
     GST_LOG_OBJECT (souphttpsink, "main loop thread running");
+
+    souphttpsink->session =
+        soup_session_async_new_with_options (SOUP_SESSION_ASYNC_CONTEXT,
+        souphttpsink->context, SOUP_SESSION_USER_AGENT,
+        souphttpsink->user_agent, SOUP_SESSION_TIMEOUT, souphttpsink->timeout,
+        NULL);
+
+    g_signal_connect (souphttpsink->session, "authenticate",
+        G_CALLBACK (authenticate), souphttpsink);
   }
 
   /* Set up logging */
   gst_soup_util_log_setup (souphttpsink->session, souphttpsink->log_level,
-      G_OBJECT (souphttpsink));
+      GST_ELEMENT (souphttpsink));
 
   return TRUE;
 }
@@ -612,23 +556,12 @@ gst_soup_http_client_sink_stop (GstBaseSink * sink)
   GST_DEBUG ("stop");
 
   if (souphttpsink->prop_session == NULL) {
-    _soup_session_abort (souphttpsink->session);
+    soup_session_abort (souphttpsink->session);
     g_object_unref (souphttpsink->session);
   }
 
-  g_mutex_lock (&souphttpsink->mutex);
-  if (souphttpsink->timer) {
-    g_source_destroy (souphttpsink->timer);
-    g_source_unref (souphttpsink->timer);
-    souphttpsink->timer = NULL;
-  }
-  g_mutex_unlock (&souphttpsink->mutex);
-
   if (souphttpsink->loop) {
     g_main_loop_quit (souphttpsink->loop);
-    g_mutex_lock (&souphttpsink->mutex);
-    g_cond_signal (&souphttpsink->cond);
-    g_mutex_unlock (&souphttpsink->mutex);
     g_thread_join (souphttpsink->thread);
     g_main_loop_unref (souphttpsink->loop);
     souphttpsink->loop = NULL;
@@ -651,13 +584,51 @@ gst_soup_http_client_sink_unlock (GstBaseSink * sink)
   return TRUE;
 }
 
+static gboolean
+gst_soup_http_client_sink_event (GstBaseSink * sink, GstEvent * event)
+{
+  GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (sink);
+
+  GST_DEBUG_OBJECT (souphttpsink, "event");
+
+  if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+    GST_DEBUG_OBJECT (souphttpsink, "got eos");
+    g_mutex_lock (&souphttpsink->mutex);
+    while (souphttpsink->message) {
+      GST_DEBUG_OBJECT (souphttpsink, "waiting");
+      g_cond_wait (&souphttpsink->cond, &souphttpsink->mutex);
+    }
+    g_mutex_unlock (&souphttpsink->mutex);
+    GST_DEBUG_OBJECT (souphttpsink, "finished eos");
+  }
+
+  return GST_BASE_SINK_CLASS (parent_class)->event (sink, event);
+}
+
+static GstFlowReturn
+gst_soup_http_client_sink_preroll (GstBaseSink * sink, GstBuffer * buffer)
+{
+  GST_DEBUG ("preroll");
+
+  return GST_FLOW_OK;
+}
+
+static void
+free_buffer_list (GList * list)
+{
+  GList *g;
+  for (g = list; g; g = g_list_next (g)) {
+    GstBuffer *buffer = g->data;
+    gst_buffer_unref (buffer);
+  }
+  g_list_free (list);
+}
+
 static void
 send_message_locked (GstSoupHttpClientSink * souphttpsink)
 {
   GList *g;
   guint64 n;
-  GByteArray *array;
-  GInputStream *in_stream;
 
   if (souphttpsink->queued_buffers == NULL || souphttpsink->message) {
     return;
@@ -665,47 +636,23 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
 
   /* If the URI went away, drop all these buffers */
   if (souphttpsink->location == NULL) {
-    GST_DEBUG_OBJECT (souphttpsink, "URI went away, dropping queued buffers");
-    g_list_free_full (souphttpsink->queued_buffers,
-        (GDestroyNotify) gst_buffer_unref);
+    free_buffer_list (souphttpsink->queued_buffers);
     souphttpsink->queued_buffers = NULL;
     return;
   }
 
-  souphttpsink->message = _soup_message_new ("PUT", souphttpsink->location);
-  if (souphttpsink->message == NULL) {
-    GST_WARNING_OBJECT (souphttpsink,
-        "URI could not be parsed while creating message.");
-    g_list_free_full (souphttpsink->queued_buffers,
-        (GDestroyNotify) gst_buffer_unref);
-    souphttpsink->queued_buffers = NULL;
-    return;
-  }
+  souphttpsink->message = soup_message_new ("PUT", souphttpsink->location);
 
-  g_signal_connect (souphttpsink->message, "restarted", G_CALLBACK (restarted),
-      souphttpsink->request_body);
-
-  _soup_message_set_flags (souphttpsink->message,
-      (souphttpsink->automatic_redirect ? 0 : SOUP_MESSAGE_NO_REDIRECT));
-
-  if (souphttpsink->cookies) {
-    gchar **cookie;
-
-    for (cookie = souphttpsink->cookies; *cookie != NULL; cookie++) {
-      _soup_message_headers_append (_soup_message_get_request_headers
-          (souphttpsink->message), "Cookie", *cookie);
-    }
-  }
-  array = g_byte_array_new ();
   n = 0;
   if (souphttpsink->offset == 0) {
     for (g = souphttpsink->streamheader_buffers; g; g = g_list_next (g)) {
       GstBuffer *buffer = g->data;
       GstMapInfo map;
 
-      GST_DEBUG_OBJECT (souphttpsink, "queueing stream headers");
+      /* FIXME, lifetime of the buffer? */
       gst_buffer_map (buffer, &map, GST_MAP_READ);
-      g_byte_array_append (array, map.data, map.size);
+      soup_message_body_append (souphttpsink->message->request_body,
+          SOUP_MEMORY_STATIC, map.data, map.size);
       n += map.size;
       gst_buffer_unmap (buffer, &map);
     }
@@ -716,63 +663,41 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
     if (!GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_HEADER)) {
       GstMapInfo map;
 
+      /* FIXME, lifetime of the buffer? */
       gst_buffer_map (buffer, &map, GST_MAP_READ);
-      g_byte_array_append (array, map.data, map.size);
+      soup_message_body_append (souphttpsink->message->request_body,
+          SOUP_MEMORY_STATIC, map.data, map.size);
       n += map.size;
       gst_buffer_unmap (buffer, &map);
     }
   }
 
-  {
-    souphttpsink->request_body = g_byte_array_free_to_bytes (array);
-    _soup_message_set_request_body_from_bytes (souphttpsink->message,
-        NULL, souphttpsink->request_body);
-  }
-
   if (souphttpsink->offset != 0) {
     char *s;
     s = g_strdup_printf ("bytes %" G_GUINT64_FORMAT "-%" G_GUINT64_FORMAT "/*",
         souphttpsink->offset, souphttpsink->offset + n - 1);
-    _soup_message_headers_append (_soup_message_get_request_headers
-        (souphttpsink->message), "Content-Range", s);
+    soup_message_headers_append (souphttpsink->message->request_headers,
+        "Content-Range", s);
     g_free (s);
   }
 
   if (n == 0) {
-    GST_DEBUG_OBJECT (souphttpsink,
-        "total size of buffers queued is 0, freeing everything");
-    g_list_free_full (souphttpsink->queued_buffers,
-        (GDestroyNotify) gst_buffer_unref);
+    free_buffer_list (souphttpsink->queued_buffers);
     souphttpsink->queued_buffers = NULL;
-    g_clear_object (&souphttpsink->message);
-    g_clear_pointer (&souphttpsink->request_body, g_bytes_unref);
+    g_object_unref (souphttpsink->message);
+    souphttpsink->message = NULL;
     return;
   }
 
-  in_stream =
-      _soup_session_send (souphttpsink->session, souphttpsink->message, NULL,
-      NULL);
-  if (in_stream == NULL) {
-    GError *error = NULL;
-
-    if (!send_handle_status (souphttpsink->message, error, souphttpsink)) {
-      g_object_unref (souphttpsink->message);
-      g_clear_pointer (&souphttpsink->request_body, g_bytes_unref);
-      g_clear_error (&error);
-      return;
-    }
-  }
   souphttpsink->sent_buffers = souphttpsink->queued_buffers;
-
-  g_clear_pointer (&souphttpsink->request_body, g_bytes_unref);
-  g_object_unref (in_stream);
-
-  g_list_free_full (souphttpsink->sent_buffers,
-      (GDestroyNotify) gst_buffer_unref);
-  souphttpsink->sent_buffers = NULL;
-  souphttpsink->failures = 0;
   souphttpsink->queued_buffers = NULL;
-  g_clear_object (&souphttpsink->message);
+
+  GST_DEBUG_OBJECT (souphttpsink,
+      "queue message %" G_GUINT64_FORMAT " %" G_GUINT64_FORMAT,
+      souphttpsink->offset, n);
+  soup_session_queue_message (souphttpsink->session, souphttpsink->message,
+      callback, souphttpsink);
+
   souphttpsink->offset += n;
 }
 
@@ -781,77 +706,35 @@ send_message (GstSoupHttpClientSink * souphttpsink)
 {
   g_mutex_lock (&souphttpsink->mutex);
   send_message_locked (souphttpsink);
-  if (souphttpsink->timer) {
-    g_source_destroy (souphttpsink->timer);
-    g_source_unref (souphttpsink->timer);
-    souphttpsink->timer = NULL;
-  }
   g_mutex_unlock (&souphttpsink->mutex);
 
   return FALSE;
 }
 
-static gboolean
-send_handle_status (SoupMessage * msg, GError * error,
-    GstSoupHttpClientSink * sink)
+static void
+callback (SoupSession * session, SoupMessage * msg, gpointer user_data)
 {
-  if (error) {
-    GST_DEBUG_OBJECT (sink, "callback error=%d %s",
-        error->code, error->message);
-  } else {
-    GST_DEBUG_OBJECT (sink, "callback status=%d %s",
-        _soup_message_get_status (msg), _soup_message_get_reason_phrase (msg));
+  GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (user_data);
+
+  GST_DEBUG_OBJECT (souphttpsink, "callback status=%d %s",
+      msg->status_code, msg->reason_phrase);
+
+  g_mutex_lock (&souphttpsink->mutex);
+  g_cond_signal (&souphttpsink->cond);
+  souphttpsink->message = NULL;
+
+  if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+    souphttpsink->status_code = msg->status_code;
+    souphttpsink->reason_phrase = g_strdup (msg->reason_phrase);
+    g_mutex_unlock (&souphttpsink->mutex);
+    return;
   }
 
-  if (error || !SOUP_STATUS_IS_SUCCESSFUL (_soup_message_get_status (msg))) {
-    sink->failures++;
-    if (sink->retries && (sink->retries < 0 || sink->retries >= sink->failures)) {
-      guint64 retry_delay;
-      const char *retry_after;
-      SoupMessageHeaders *res_hdrs;
-      if (error) {
-        retry_delay = sink->retry_delay;
-        GST_WARNING_OBJECT (sink, "Could not write to HTTP URI: "
-            "error: %d %s (retrying PUT after %" G_GINT64_FORMAT
-            " seconds)", error->code, error->message, retry_delay);
-        goto err_done;
-      }
-      res_hdrs = _soup_message_get_response_headers (msg);
-      retry_after = _soup_message_headers_get_one (res_hdrs, "Retry-After");
-      if (retry_after) {
-        gchar *end = NULL;
-        retry_delay = g_ascii_strtoull (retry_after, &end, 10);
-        if (end || errno) {
-          retry_delay = sink->retry_delay;
-        } else {
-          retry_delay = MAX (retry_delay, sink->retry_delay);
-        }
-        GST_WARNING_OBJECT (sink, "Could not write to HTTP URI: "
-            "status: %d %s (retrying PUT after %" G_GINT64_FORMAT
-            " seconds with Retry-After: %s)",
-            _soup_message_get_status (msg),
-            _soup_message_get_reason_phrase (msg), retry_delay, retry_after);
-      } else {
-        retry_delay = sink->retry_delay;
-        GST_WARNING_OBJECT (sink, "Could not write to HTTP URI: "
-            "status: %d %s (retrying PUT after %" G_GINT64_FORMAT
-            " seconds)",
-            _soup_message_get_status (msg),
-            _soup_message_get_reason_phrase (msg), retry_delay);
-      }
-    err_done:
-      sink->timer = g_timeout_source_new_seconds (retry_delay);
-      g_source_set_callback (sink->timer, (GSourceFunc) (send_message),
-          sink, NULL);
-      g_source_attach (sink->timer, sink->context);
-    } else {
-      sink->status_code = _soup_message_get_status (msg);
-      sink->reason_phrase = g_strdup (_soup_message_get_reason_phrase (msg));
-    }
-    return FALSE;
-  }
+  free_buffer_list (souphttpsink->sent_buffers);
+  souphttpsink->sent_buffers = NULL;
 
-  return TRUE;
+  send_message_locked (souphttpsink);
+  g_mutex_unlock (&souphttpsink->mutex);
 }
 
 static GstFlowReturn
@@ -859,26 +742,30 @@ gst_soup_http_client_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 {
   GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (sink);
   GSource *source;
+  gboolean wake;
 
   if (souphttpsink->status_code != 0) {
+    /* FIXME we should allow a moderate amount of retries. */
     GST_ELEMENT_ERROR (souphttpsink, RESOURCE, WRITE,
         ("Could not write to HTTP URI"),
-        ("status: %d %s", souphttpsink->status_code,
+        ("error: %d %s", souphttpsink->status_code,
             souphttpsink->reason_phrase));
     return GST_FLOW_ERROR;
   }
 
   g_mutex_lock (&souphttpsink->mutex);
   if (souphttpsink->location != NULL) {
+    wake = (souphttpsink->queued_buffers == NULL);
     souphttpsink->queued_buffers =
         g_list_append (souphttpsink->queued_buffers, gst_buffer_ref (buffer));
 
-    GST_DEBUG_OBJECT (souphttpsink, "setting callback for new buffers");
-    source = g_idle_source_new ();
-    g_source_set_callback (source, (GSourceFunc) (send_message),
-        souphttpsink, NULL);
-    g_source_attach (source, souphttpsink->context);
-    g_source_unref (source);
+    if (wake) {
+      source = g_idle_source_new ();
+      g_source_set_callback (source, (GSourceFunc) (send_message),
+          souphttpsink, NULL);
+      g_source_attach (source, souphttpsink->context);
+      g_source_unref (source);
+    }
   }
   g_mutex_unlock (&souphttpsink->mutex);
 
@@ -886,47 +773,15 @@ gst_soup_http_client_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 }
 
 static void
-restarted (SoupMessage * msg, GBytes * body)
-{
-  _soup_message_set_request_body_from_bytes (msg, NULL, body);
-}
-
-static gboolean
-authenticate (SoupMessage * msg, SoupAuth * auth,
-    gboolean retrying, gpointer user_data)
+authenticate (SoupSession * session, SoupMessage * msg,
+    SoupAuth * auth, gboolean retrying, gpointer user_data)
 {
   GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (user_data);
 
   if (!retrying) {
-    SoupStatus status_code = _soup_message_get_status (msg);
-    /* First time authentication only, if we fail and are called again with retry true fall through */
-    if (status_code == SOUP_STATUS_UNAUTHORIZED) {
-      if (souphttpsink->user_id && souphttpsink->user_pw)
-        _soup_auth_authenticate (auth, souphttpsink->user_id,
-            souphttpsink->user_pw);
-    } else if (status_code == SOUP_STATUS_PROXY_AUTHENTICATION_REQUIRED) {
-      if (souphttpsink->proxy_id && souphttpsink->proxy_pw)
-        _soup_auth_authenticate (auth, souphttpsink->proxy_id,
-            souphttpsink->proxy_pw);
+    if (souphttpsink->user_id && souphttpsink->user_pw) {
+      soup_auth_authenticate (auth,
+          souphttpsink->user_id, souphttpsink->user_pw);
     }
   }
-  return FALSE;
-}
-
-static gboolean
-souphttpclientsink_element_init (GstPlugin * plugin)
-{
-  gboolean ret = TRUE;
-
-  GST_DEBUG_CATEGORY_INIT (souphttpclientsink_dbg, "souphttpclientsink", 0,
-      "souphttpclientsink element");
-
-  if (!soup_element_init (plugin))
-    return TRUE;
-
-  ret =
-      gst_element_register (plugin, "souphttpclientsink", GST_RANK_NONE,
-      GST_TYPE_SOUP_HTTP_CLIENT_SINK);
-
-  return ret;
 }
