@@ -106,7 +106,7 @@ static gboolean gst_vp9_dec_start (GstVideoDecoder * decoder);
 static gboolean gst_vp9_dec_stop (GstVideoDecoder * decoder);
 static gboolean gst_vp9_dec_set_format (GstVideoDecoder * decoder,
     GstVideoCodecState * state);
-static gboolean gst_vp9_dec_reset (GstVideoDecoder * decoder, gboolean hard);
+static gboolean gst_vp9_dec_flush (GstVideoDecoder * decoder);
 static GstFlowReturn gst_vp9_dec_handle_frame (GstVideoDecoder * decoder,
     GstVideoCodecFrame * frame);
 static gboolean gst_vp9_dec_decide_allocation (GstVideoDecoder * decoder,
@@ -184,7 +184,7 @@ gst_vp9_dec_class_init (GstVP9DecClass * klass)
 
   base_video_decoder_class->start = GST_DEBUG_FUNCPTR (gst_vp9_dec_start);
   base_video_decoder_class->stop = GST_DEBUG_FUNCPTR (gst_vp9_dec_stop);
-  base_video_decoder_class->reset = GST_DEBUG_FUNCPTR (gst_vp9_dec_reset);
+  base_video_decoder_class->flush = GST_DEBUG_FUNCPTR (gst_vp9_dec_flush);
   base_video_decoder_class->set_format =
       GST_DEBUG_FUNCPTR (gst_vp9_dec_set_format);
   base_video_decoder_class->handle_frame =
@@ -205,6 +205,8 @@ gst_vp9_dec_init (GstVP9Dec * gst_vp9_dec)
   gst_vp9_dec->post_processing_flags = DEFAULT_POST_PROCESSING_FLAGS;
   gst_vp9_dec->deblocking_level = DEFAULT_DEBLOCKING_LEVEL;
   gst_vp9_dec->noise_level = DEFAULT_NOISE_LEVEL;
+
+  gst_video_decoder_set_needs_format (decoder, TRUE);
 }
 
 static void
@@ -324,11 +326,11 @@ gst_vp9_dec_set_format (GstVideoDecoder * decoder, GstVideoCodecState * state)
 }
 
 static gboolean
-gst_vp9_dec_reset (GstVideoDecoder * base_video_decoder, gboolean hard)
+gst_vp9_dec_flush (GstVideoDecoder * base_video_decoder)
 {
   GstVP9Dec *decoder;
 
-  GST_DEBUG_OBJECT (base_video_decoder, "reset");
+  GST_DEBUG_OBJECT (base_video_decoder, "flush");
 
   decoder = GST_VP9_DEC (base_video_decoder);
 
