@@ -22,6 +22,12 @@
 #ifndef __GST_VP9_ENC_H__
 #define __GST_VP9_ENC_H__
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#ifdef HAVE_VP9_ENCODER
+
 #include <gst/gst.h>
 #include <gst/video/gstvideoencoder.h>
 
@@ -110,4 +116,6 @@ GType gst_vp9_enc_get_type (void);
 
 G_END_DECLS
 
+#endif
+
 #endif /* __GST_VP9_ENC_H__ */
