@@ -20,7 +20,6 @@
 
 /**
  * SECTION:element-audioecho
- * @Since: 0.10.14
  *
  * audioecho adds an echo or (simple) reverb effect to an audio stream. The echo
  * delay, intensity and the percentage of feedback can be configured.
@@ -187,6 +186,7 @@ gst_audio_echo_set_property (GObject * object, guint prop_id,
   switch (prop_id) {
     case PROP_DELAY:{
       guint64 max_delay, delay;
+      guint rate;
 
       g_mutex_lock (&self->lock);
       delay = g_value_get_uint64 (value);
@@ -201,6 +201,11 @@ gst_audio_echo_set_property (GObject * object, guint prop_id,
         self->delay = delay;
         self->max_delay = MAX (delay, max_delay);
       }
+      rate = GST_AUDIO_FILTER_RATE (self);
+      if (rate > 0)
+        self->delay_frames =
+            MAX (gst_util_uint64_scale (self->delay, rate, GST_SECOND), 1);
+
       g_mutex_unlock (&self->lock);
       break;
     }
