@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* TODO:
@@ -217,11 +217,13 @@ static gboolean gst_interleave_src_event (GstPad * pad, GstObject * parent,
 
 static gboolean gst_interleave_sink_event (GstCollectPads * pads,
     GstCollectData * data, GstEvent * event, gpointer user_data);
+static gboolean gst_interleave_sink_query (GstCollectPads * pads,
+    GstCollectData * data, GstQuery * query, gpointer user_data);
 
 static gboolean gst_interleave_sink_setcaps (GstInterleave * self,
     GstPad * pad, const GstCaps * caps, const GstAudioInfo * info);
 
-static GstCaps *gst_interleave_sink_getcaps (GstPad * pad, GstObject * parent,
+static GstCaps *gst_interleave_sink_getcaps (GstPad * pad, GstInterleave * self,
     GstCaps * filter);
 
 static GstFlowReturn gst_interleave_collected (GstCollectPads * pads,
@@ -296,6 +298,24 @@ gst_interleave_set_channel_positions (GstInterleave * self, GstStructure * s)
   gst_structure_set (s, "channel-mask", GST_TYPE_BITMASK, channel_mask, NULL);
 }
 
+static void
+gst_interleave_send_stream_start (GstInterleave * self)
+{
+  GST_OBJECT_LOCK (self);
+  if (self->send_stream_start) {
+    gchar s_id[32];
+
+    self->send_stream_start = FALSE;
+    GST_OBJECT_UNLOCK (self);
+
+    /* stream-start (FIXME: create id based on input ids) */
+    g_snprintf (s_id, sizeof (s_id), "interleave-%08x", g_random_int ());
+    gst_pad_push_event (self->src, gst_event_new_stream_start (s_id));
+  } else {
+    GST_OBJECT_UNLOCK (self);
+  }
+}
+
 static void
 gst_interleave_class_init (GstInterleaveClass * klass)
 {
@@ -386,7 +406,6 @@ gst_interleave_init (GstInterleave * self)
   gst_pad_set_event_function (self->src,
       GST_DEBUG_FUNCPTR (gst_interleave_src_event));
 
-  gst_pad_set_active (self->src, TRUE);
   gst_element_add_pad (GST_ELEMENT (self), self->src);
 
   self->collect = gst_collect_pads_new ();
@@ -485,6 +504,10 @@ gst_interleave_request_new_pad (GstElement * element, GstPadTemplate * templ,
       (GstCollectPadsEventFunction)
       GST_DEBUG_FUNCPTR (gst_interleave_sink_event), self);
 
+  gst_collect_pads_set_query_function (self->collect,
+      (GstCollectPadsQueryFunction)
+      GST_DEBUG_FUNCPTR (gst_interleave_sink_query), self);
+
   if (!gst_element_add_pad (element, new_pad))
     goto could_not_add;
 
@@ -508,8 +531,7 @@ gst_interleave_request_new_pad (GstElement * element, GstPadTemplate * templ,
     gst_structure_set (s, "channels", G_TYPE_INT, self->channels, NULL);
     gst_interleave_set_channel_positions (self, s);
 
-    /* FIXME: send caps event after stream-start event */
-    gst_pad_set_active (self->src, TRUE);
+    gst_interleave_send_stream_start (self);
     gst_pad_set_caps (self->src, srccaps);
     gst_caps_unref (srccaps);
 
@@ -575,7 +597,7 @@ gst_interleave_release_pad (GstElement * element, GstPad * pad)
       gst_structure_set (s, "channels", G_TYPE_INT, self->channels, NULL);
       gst_interleave_set_channel_positions (self, s);
 
-      gst_pad_set_active (self->src, TRUE);
+      gst_interleave_send_stream_start (self);
       gst_pad_set_caps (self->src, srccaps);
       gst_caps_unref (srccaps);
     } else {
@@ -670,9 +692,9 @@ __set_channels (GstCaps * caps, gint channels)
 
 /* we can only accept caps that we and downstream can handle. */
 static GstCaps *
-gst_interleave_sink_getcaps (GstPad * pad, GstObject * parent, GstCaps * filter)
+gst_interleave_sink_getcaps (GstPad * pad, GstInterleave * self,
+    GstCaps * filter)
 {
-  GstInterleave *self = GST_INTERLEAVE (parent);
   GstCaps *result, *peercaps, *sinkcaps;
 
   GST_OBJECT_LOCK (self);
@@ -688,6 +710,7 @@ gst_interleave_sink_getcaps (GstPad * pad, GstObject * parent, GstCaps * filter)
     sinkcaps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
     __remove_channels (sinkcaps);
     if (peercaps) {
+      peercaps = gst_caps_make_writable (peercaps);
       __remove_channels (peercaps);
       /* if the peer has caps, intersect */
       GST_DEBUG_OBJECT (pad, "intersecting peer and template caps");
@@ -762,7 +785,7 @@ gst_interleave_sink_setcaps (GstInterleave * self, GstPad * pad,
     gst_structure_set (s, "channels", G_TYPE_INT, self->channels, NULL);
     gst_interleave_set_channel_positions (self, s);
 
-    gst_pad_set_active (self->src, TRUE);
+    gst_interleave_send_stream_start (self);
     res = gst_pad_set_caps (self->src, srccaps);
     gst_caps_unref (srccaps);
 
@@ -871,6 +894,36 @@ gst_interleave_sink_event (GstCollectPads * pads, GstCollectData * data,
   return ret;
 }
 
+static gboolean
+gst_interleave_sink_query (GstCollectPads * pads,
+    GstCollectData * data, GstQuery * query, gpointer user_data)
+{
+  GstInterleave *self = GST_INTERLEAVE (user_data);
+  gboolean ret = TRUE;
+
+  GST_DEBUG ("Got %s query on pad %s:%s", GST_QUERY_TYPE_NAME (query),
+      GST_DEBUG_PAD_NAME (data->pad));
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CAPS:
+    {
+      GstCaps *filter, *caps;
+
+      gst_query_parse_caps (query, &filter);
+      caps = gst_interleave_sink_getcaps (data->pad, self, filter);
+      gst_query_set_caps_result (query, caps);
+      gst_caps_unref (caps);
+      ret = TRUE;
+      break;
+    }
+    default:
+      ret = gst_collect_pads_query_default (pads, data, query, FALSE);
+      break;
+  }
+
+  return ret;
+}
+
 static gboolean
 gst_interleave_src_query_duration (GstInterleave * self, GstQuery * query)
 {
@@ -1074,16 +1127,6 @@ gst_interleave_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
     case GST_QUERY_LATENCY:
       res = gst_interleave_src_query_latency (self, query);
       break;
-    case GST_QUERY_CAPS:
-    {
-      GstCaps *filter, *caps;
-
-      gst_query_parse_caps (query, &filter);
-      caps = gst_interleave_sink_getcaps (pad, parent, filter);
-      gst_query_set_caps_result (query, caps);
-      gst_caps_unref (caps);
-      res = TRUE;
-    }
     default:
       /* FIXME, needs a custom query handler because we have multiple
        * sinkpads */
@@ -1189,18 +1232,6 @@ gst_interleave_collected (GstCollectPads * pads, GstInterleave * self)
   GstMapInfo write_info;
   GstClockTime timestamp = -1;
 
-  /* FIXME: send caps and tags after stream-start */
-#if 0
-  if (self->send_stream_start) {
-    gchar s_id[32];
-
-    /* stream-start (FIXME: create id based on input ids) */
-    g_snprintf (s_id, sizeof (s_id), "interleave-%08x", g_random_int ());
-    gst_pad_push_event (self->src, gst_event_new_stream_start (s_id));
-    self->send_stream_start = FALSE;
-  }
-#endif
-
   size = gst_collect_pads_available (pads);
   if (size == 0)
     goto eos;
