@@ -23,6 +23,7 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstadapter.h>
+#include <gst/base/gstflowcombiner.h>
 
 G_BEGIN_DECLS
 
@@ -64,6 +65,8 @@ struct _GstQTDemux {
   gint     n_audio_streams;
   gint     n_sub_streams;
 
+  GstFlowCombiner *flowcombiner;
+
   gboolean have_group_id;
   guint group_id;
 
@@ -91,12 +94,22 @@ struct _GstQTDemux {
   GstAdapter *adapter;
   GstBuffer *mdatbuffer;
   guint64 mdatleft;
+  /* When restoring the mdat to the adatpter, this buffer
+   * stores any trailing data that was after the last atom parsed as it
+   * has to be restored later along with the correct offset. Used in
+   * fragmented scenario where mdat/moof are one after the other
+   * in any order.
+   *
+   * Check https://bugzilla.gnome.org/show_bug.cgi?id=710623 */
+  GstBuffer *restoredata_buffer;
+  guint64 restoredata_offset;
 
   guint64 offset;
   /* offset of the mdat atom */
   guint64 mdatoffset;
   guint64 first_mdat;
   gboolean got_moov;
+  guint64 last_moov_offset;
   guint header_size;
 
   GstTagList *tag_list;
@@ -104,10 +117,16 @@ struct _GstQTDemux {
   /* configured playback region */
   GstSegment segment;
   GstEvent *pending_newsegment;
-  gboolean upstream_newsegment;
+  gboolean upstream_newsegment; /* qtdemux received upstream
+                                 * newsegment in TIME format which likely
+                                 * means that upstream is driving the pipeline
+                                 * (adaptive demuxers) */
   gint64 seek_offset;
   gint64 push_seek_start;
   gint64 push_seek_stop;
+  guint64 segment_base; /* The offset from which playback was started, needs to
+                         * be subtracted from GstSegment.base to get a correct
+                         * running time whenever a new QtSegment is activated */
 
 #if 0
   /* gst index support */
@@ -122,12 +141,13 @@ struct _GstQTDemux {
    * upstream provides it at the caps */
   GstCaps *media_caps;
   gboolean exposed;
-  gboolean mss_mode; /* flag to indicate that we're working with a smoothstreaming fragment */
+  gboolean mss_mode; /* flag to indicate that we're working with a smoothstreaming fragment
+                      * Mss doesn't have 'moov' or any information about the streams format,
+                      * requiring qtdemux to expose and create the streams */
   guint64 fragment_start;
+  guint64 fragment_start_offset;
     
   gint64 chapters_track_id;
-
-  GstClockTime min_elst_offset;
 };
 
 struct _GstQTDemuxClass {
