@@ -171,7 +171,7 @@ gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
   GstRtpVRawDepay *rtpvrawdepay;
   gint clock_rate;
   const gchar *str;
-  gint format, width, height, pgroup, xinc, yinc;
+  gint format, width, height, depth, pgroup, xinc, yinc;
   GstCaps *srccaps;
   gboolean res;
   GstFlowReturn ret;
@@ -194,6 +194,10 @@ gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
     goto no_height;
   height = atoi (str);
 
+  if (!(str = gst_structure_get_string (structure, "depth")))
+    goto no_depth;
+  depth = atoi (str);
+
   /* optional interlace value but we don't handle interlaced
    * formats yet */
   if (gst_structure_get_string (structure, "interlace"))
@@ -218,8 +222,14 @@ gst_rtp_vraw_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
     format = GST_VIDEO_FORMAT_AYUV;
     pgroup = 3;
   } else if (!strcmp (str, "YCbCr-4:2:2")) {
-    format = GST_VIDEO_FORMAT_UYVY;
-    pgroup = 4;
+    if (depth == 8) {
+      format = GST_VIDEO_FORMAT_UYVY;
+      pgroup = 4;
+    } else if (depth == 10) {
+      format = GST_VIDEO_FORMAT_UYVP;
+      pgroup = 5;
+    } else
+      goto unknown_format;
     xinc = 2;
   } else if (!strcmp (str, "YCbCr-4:2:0")) {
     format = GST_VIDEO_FORMAT_I420;
@@ -268,6 +278,11 @@ no_height:
     GST_ERROR_OBJECT (depayload, "no height specified");
     return FALSE;
   }
+no_depth:
+  {
+    GST_ERROR_OBJECT (depayload, "no depth specified");
+    return FALSE;
+  }
 interlaced:
   {
     GST_ERROR_OBJECT (depayload, "interlaced formats not supported yet");
@@ -294,7 +309,7 @@ static GstBuffer *
 gst_rtp_vraw_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
 {
   GstRtpVRawDepay *rtpvrawdepay;
-  guint8 *payload, *yp, *up, *vp, *headers;
+  guint8 *payload, *p0, *yp, *up, *vp, *headers;
   guint32 timestamp;
   guint cont, ystride, uvstride, pgroup, payload_len;
   gint width, height, xinc, yinc;
@@ -346,6 +361,7 @@ gst_rtp_vraw_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
     goto invalid_frame;
 
   /* get pointer and strides of the planes */
+  p0 = GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
   yp = GST_VIDEO_FRAME_COMP_DATA (&frame, 0);
   up = GST_VIDEO_FRAME_COMP_DATA (&frame, 1);
   vp = GST_VIDEO_FRAME_COMP_DATA (&frame, 2);
@@ -434,9 +450,10 @@ gst_rtp_vraw_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
       case GST_VIDEO_FORMAT_BGR:
       case GST_VIDEO_FORMAT_BGRA:
       case GST_VIDEO_FORMAT_UYVY:
+      case GST_VIDEO_FORMAT_UYVP:
         /* samples are packed just like gstreamer packs them */
         offs /= xinc;
-        datap = yp + (line * ystride) + (offs * pgroup);
+        datap = p0 + (line * ystride) + (offs * pgroup);
 
         memcpy (datap, payload, plen);
         break;
@@ -445,7 +462,7 @@ gst_rtp_vraw_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
         gint i;
         guint8 *p;
 
-        datap = yp + (line * ystride) + (offs * 4);
+        datap = p0 + (line * ystride) + (offs * 4);
         p = payload;
 
         /* samples are packed in order Cb-Y-Cr for both interlaced and
