@@ -58,6 +58,7 @@ struct _GstRtpBin {
   gboolean        use_pipeline_clock;
   gboolean        send_sync_event;
   GstClockTime    buffer_start;
+  gboolean        do_retransmission;
   /* a list of session */
   GSList         *sessions;
 
@@ -79,10 +80,12 @@ struct _GstRtpBinClass {
 
   void        (*payload_type_change)  (GstRtpBin *rtpbin, guint session, guint pt);
 
+  void        (*new_jitterbuffer)     (GstRtpBin *rtpbin, guint session, guint32 ssrc);
+
   /* action signals */
   void        (*clear_pt_map)         (GstRtpBin *rtpbin);
   void        (*reset_sync)           (GstRtpBin *rtpbin);
-  RTPSession* (*get_internal_session) (GstRtpBin *rtpbin, guint session_id);
+  RTPSession* (*get_internal_session) (GstRtpBin *rtpbin, guint session);
 
   /* session manager signals */
   void     (*on_new_ssrc)       (GstRtpBin *rtpbin, guint session, guint32 ssrc);
@@ -95,6 +98,14 @@ struct _GstRtpBinClass {
   void     (*on_timeout)        (GstRtpBin *rtpbin, guint session, guint32 ssrc);
   void     (*on_sender_timeout) (GstRtpBin *rtpbin, guint session, guint32 ssrc);
   void     (*on_npt_stop)       (GstRtpBin *rtpbin, guint session, guint32 ssrc);
+
+  GstElement* (*request_rtp_encoder)  (GstRtpBin *rtpbin, guint session);
+  GstElement* (*request_rtp_decoder)  (GstRtpBin *rtpbin, guint session);
+  GstElement* (*request_rtcp_encoder) (GstRtpBin *rtpbin, guint session);
+  GstElement* (*request_rtcp_decoder) (GstRtpBin *rtpbin, guint session);
+
+  GstElement* (*request_aux_sender)   (GstRtpBin *rtpbin, guint session);
+  GstElement* (*request_aux_receiver) (GstRtpBin *rtpbin, guint session);
 };
 
 GType gst_rtp_bin_get_type (void);
