@@ -27,6 +27,7 @@
 #ifndef __GST_OSX_VIDEO_SINK_H__
 #define __GST_OSX_VIDEO_SINK_H__
 
+#include <gst/video/video.h>
 #include <gst/video/gstvideosink.h>
 
 #include <string.h>
@@ -40,14 +41,6 @@
 GST_DEBUG_CATEGORY_EXTERN (gst_debug_osx_video_sink);
 #define GST_CAT_DEFAULT gst_debug_osx_video_sink
 
-/* The hack doesn't work on leopard, the _CFMainPThread symbol
- * is doesn't exist in the CoreFoundation library */
-#if MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_5
-#ifdef RUN_NS_APP_THREAD
-#undef RUN_NS_APP_THREAD
-#endif
-#endif
-
 G_BEGIN_DECLS
 
 #define GST_TYPE_OSX_VIDEO_SINK \
@@ -68,6 +61,12 @@ typedef struct _GstOSXVideoSinkClass GstOSXVideoSinkClass;
 
 #define GST_TYPE_OSXVIDEOBUFFER (gst_osxvideobuffer_get_type())
 
+typedef enum {
+  GST_OSX_VIDEO_SINK_RUN_LOOP_STATE_NOT_RUNNING = 0,
+  GST_OSX_VIDEO_SINK_RUN_LOOP_STATE_RUNNING = 1,
+  GST_OSX_VIDEO_SINK_RUN_LOOP_STATE_UNKNOWN = 2,
+} GstOSXVideoSinkRunLoopState;
+
 /* OSXWindow stuff */
 struct _GstOSXWindow {
   gint width, height;
@@ -83,33 +82,22 @@ struct _GstOSXVideoSink {
   GstOSXWindow *osxwindow;
   void *osxvideosinkobject;
   NSView *superview;
-  NSThread *ns_app_thread;
-#ifdef RUN_NS_APP_THREAD
-  GMutex loop_thread_lock;
-  GCond loop_thread_cond;
-#else
-  guint cocoa_timeout;
-#endif
-  GMutex mrl_check_lock;
-  GCond mrl_check_cond;
-  gboolean mrl_check_done;
-  gboolean main_run_loop_running;
-  gboolean app_started;
   gboolean keep_par;
-  gboolean embed;
+  GstVideoInfo info;
 };
 
 struct _GstOSXVideoSinkClass {
   GstVideoSinkClass parent_class;
+
+  GstOSXVideoSinkRunLoopState run_loop_state;
+  NSThread *ns_app_thread;
 };
 
 GType gst_osx_video_sink_get_type(void);
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
 @interface NSApplication(AppleMenu)
 - (void)setAppleMenu:(NSMenu *)menu;
 @end
-#endif
 
 @interface GstBufferObject : NSObject
 {
@@ -121,11 +109,7 @@ GType gst_osx_video_sink_get_type(void);
 @end
 
 
-#if MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_5
-@interface GstWindowDelegate : NSObject
-#else
 @interface GstWindowDelegate : NSObject <NSWindowDelegate>
-#endif
 {
   @public
   GstOSXVideoSink *osxvideosink;
@@ -144,11 +128,10 @@ GType gst_osx_video_sink_get_type(void);
 -(void) resize;
 -(void) destroy;
 -(void) showFrame: (GstBufferObject*) buf;
-#ifdef RUN_NS_APP_THREAD
+-(void) setView: (NSView*) view;
 + (BOOL) isMainThread;
 -(void) nsAppThread;
 -(void) checkMainRunLoop;
-#endif
 @end
 
 G_END_DECLS
