@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +28,7 @@
 #define ASPECT_RATIO_CROP_CAPS                      \
   GST_VIDEO_CAPS_MAKE ("{ RGBx, xRGB, BGRx, xBGR, " \
       "RGBA, ARGB, BGRA, ABGR, RGB, BGR, AYUV, "    \
-      "YUY2, YVYU, UYVY, Y800, I420, YV12, RGB16, " \
+      "YUY2, YVYU, UYVY, GRAY8, I420, YV12, RGB16, " \
       "RGB15 }")
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -69,8 +69,7 @@ check_aspectratiocrop (const gchar * in_string, const gchar * out_string,
   src_pad = gst_pad_new (NULL, GST_PAD_SRC);
   gst_pad_set_active (src_pad, TRUE);
   GST_DEBUG ("setting caps %s %" GST_PTR_FORMAT, in_string, incaps);
-  fail_unless (gst_pad_set_caps (src_pad, incaps),
-      "Couldn't set input caps %" GST_PTR_FORMAT, incaps);
+  gst_check_setup_events (src_pad, element, incaps, GST_FORMAT_TIME);
 
   pad_peer = gst_element_get_static_pad (element, "sink");
   fail_if (pad_peer == NULL);
